@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
@@ -23,6 +21,7 @@
 #include "config.h"
 
 #include "dconf-shm.h"
+#include "dconf-shm-mockable.h"
 
 #include <sys/mman.h>
 #include <unistd.h>
@@ -84,7 +83,7 @@ dconf_shm_open (const gchar *name)
    * By writing to the second byte in the file we ensure we don't
    * overwrite the first byte (which is the one we care about).
    */
-  if (pwrite (fd, "", 1, 1) != 1)
+  if (dconf_shm_pwrite (fd, "", 1, 1) != 1)
     {
       g_critical ("failed to allocate file '%s': %s.  dconf will not work properly.", filename, g_strerror (errno));
       goto out;
@@ -126,7 +125,7 @@ dconf_shm_flag (const gchar *name)
        * If this fails then it will probably fail for the client too.
        * If it doesn't then there's not really much we can do...
        */
-      if (pwrite (fd, "", 1, 1) == 1)
+      if (dconf_shm_pwrite (fd, "", 1, 1) == 1)
         {
           guint8 *shm;
 
